@@ -31,7 +31,10 @@ fetchPedonPC <- function(dsn) {
 
 	# test for bad horizonation... flag, and remove
 	cat('finding horizonation errors ...\n')
-	h.test <- ddply(h, 'peiid', test_hz_logic, topcol='hzdept', bottomcol='hzdepb', strict=TRUE)
+	h.test <- ddply(h, 'peiid', function(d) {
+	  res <- aqp::hzDepthTests(top=d[['hzdept']], bottom=d[['hzdepb']])
+	  return(data.frame(hz_logic_pass=all(!res)))
+	})
 	
 	# which are the good (valid) ones?
 	good.pedon.ids <- as.character(h.test$peiid[which(h.test$hz_logic_pass)])
