@@ -6,11 +6,10 @@ get_colors_from_pedon_db <- function(dsn) {
     stop('please install the `RODBC` package', call.=FALSE)
   
 	# color data... check
-	q <- "SELECT phorizon.phiid as phiid, colormoistst, colorpct as pct, mh.choice AS colorhue, colorvalue, colorchroma
+	q <- "SELECT phorizon.phiid as phiid, colormoistst, colorpct as pct, colorhue, colorvalue, colorchroma
 FROM (
 	(pedon INNER JOIN phorizon ON pedon.peiid = phorizon.peiidref)
 	INNER JOIN phcolor ON phorizon.phiid = phcolor.phiidref)
-	LEFT OUTER JOIN (SELECT * FROM metadata_domain_detail WHERE domain_id = 1242) AS mh ON phcolor.colorhue = mh.choice_id
 	ORDER BY phorizon.phiid, colormoistst;"
   
 	# setup connection to our pedon database
@@ -21,6 +20,9 @@ FROM (
 	
 	# close connection
 	RODBC::odbcClose(channel)
+
+	# uncode domained columns
+	d <- uncode(d)
 	
 	# convert Munsell to RGB
 	cat('converting Munsell to RGB ...\n')
